*************************************************************************
* filename: RPT_SERV.PRG                                                *
* revised :  2/13/94                                                    *
*************************************************************************
* purpose: Process request for reports from REQUESTS.DBF. JFF.          *
*************************************************************************
* PREFACE: Welcome to RPT_SERV.PRG! This program allows you to run      *
*          reports concurrently with your other processing.  The        *
*          principle is simple: Another program, RQST_GNR.PRG, has       *
*          generated the necessary parameters to run a report.          *
*          RPT_SERV.PRG retrieves these parameters from the database    *
*          REQUESTS.DBF and processes the requested report from them.   *
*          The two programs, RQST_GNR.PRG and RPT_SERV.PRG, typically   *
*          run concurrently on different computers on the same          *
*          network.  After a report request is initiated by             *
*          RQST_GNR.PRG the work station is immediately available for   *
*          another task while the report processing can begin almost    *
*          immediately under the control of RPT_SERV.PRG running        *
*          on another work station. Another configuration is both       *
*          programs run on a standalone computer: RQST_GNR.PRG during   *
*          the normal work day and RPT_SERV.PRG after hours to run      *
*          reports over night.                                          *
*                                                                       *
*          RPT_SERV.PRG and RQST_GNR.PRG are necessarily incomlete      *
*          programs. You must define such things as the procedure       *
*          file, pathnames, etc. Where such items must be defined       *
*          a comment of the form **^^** is inserted to alert you        *
*          to the need to insert custom coding.                         *
*                                                                       *
*          The requests database is the file RPT_RQST.DBF:              *
*                                                                       *
*         Field Name      Type  Len Comments                            *
*           1   P_LEVEL    C      1 Priority level of the request.      *
*           2   DT_AND_TM  C     14 Date and time of the request as     *
*                                   YYYYMMDDHHMMSS (year-month-day-     *
*                                   hour-minute-second)                 *
*           3   PRG_MOD    C      8 Program module that will prepare    *
*                                   and print the report.               *
*           4   RPT_SPCS   M     10 Memo field containing the variable  *
*                                   specifications for the report.      *
*           5   DATA_PATH  C     20 Network drive resource which is     *
*                                   the default source of report data.  *
*                                                                       *
*         RPT_RQST is indexed on P_LEVEL + DT_AND_TM to RPT_RQST.IDX.   *
*                                                                       *
*         An example report included with this distribution is the      *
*         printing of a customer list.  The included database for it,   *
*         CSTMERS.DBF, has the following structure:                     *
*                                                                       *
*         Field Name      Type  Len Comments                            *
*             1 CUST_NO     C     8 Customer number
*             2 COMPANY     C    30 Customer Name                       *
*             3 ADDR1       C    25 First address line                  *
*             4 ADDR2       C    25 Second address line                 *
*             5 CITY        C    25 City                                *
*             6 STATE       C     2 State                               *
*             7 ZIP         C    10 Zip code                            *
*             8 SALES       N     2 Sales territory assigned to         *
*                                                                       *
*          CSTMERS is indexed on CUST_NO to tag CUST_NO and on          *
*                                COMPANY to tag COMPANY of CSTMERS.CDX  *
*                                                                       *
*                                                                       *
*          If you find these programs to be useful and want to          *
*          continue using them, please register with:                   *
*                                                                       *
*               COMPUTER PRODUCTS implementation                        *
*               6443 Wilkinson Avenue                                   *
*               North Hollywood,  CA  91606                             *
*                    (818) 762-0380                                     *
*          for a registration fee of $25.00 U.S.A. currency             *
*                                                                       *
*          For advice and assistance in getting your report server      *
*          in operation, feel free to contact the author: John Ferguson *
*          COMPUSERVE ID 73052,2675                                     *
*                                                                       *
*          Happy reporting with RPT_SERV.PRG and RQST_GNR.PRG!          *
*                                                                       *
PARAMETER muser         && **^^** include this line if you need to identify
                        **        the user, or any other parameter needed
                        **        by RPT_SERV.PRG throughout the run.
CLOSE ALL
* set runtime environment *
SET STATUS OFF
SET TALK OFF
SET BELL OFF
SET SCORE OFF
SET SAFETY ON
SET CONFIRM OFF
SET DATE AMERICAN
SET CENTURY OFF
** replace the definition of work_stn with an environment variable.
wrk_stn=GETENV('WRK_STN')
        **^^** include the command set wrk_stn=<name of work station>
        **^^** in your AUTOEXEC.BAT file if you need to identify the
        **^^** work station the report server is running on.
* clean up *
CLEAR PROGRAM
* activate file sharing environment *
SET EXCLUSIVE OFF
SET REPROCESS TO AUTOMATIC
SET NOTIFY ON
* activate system error handling *
dt_pth=IIF(wrk_stn $ 'PR1 PR2 MIS','D:\DATA\',;
         IIF(wrk_stn $ 'PR3 PROD','C:\DATA\',""))
         && **^^** a data path is assumed: You need to specify it
         && **^^** for your particular use.
SET ESCAPE ON
ON ESCAPE DO on_escp && **^^**  you may want to write your own trap for the
                          ** escape key press
* ON ERROR DO err_trap WITH ERROR(), MESSAGE(), MESSAGE(1), PROGRAM(), LINENO()
**^^** Insert your own error trapping module here.
**^^** replace MYPROCED below with your own procedure file, if applicable
* SET PROCEDURE TO MYPROCED
CLEAR
mco=' MY COMPANY'       && **^^** replace MY COMPANY with the name you want
DO SHOW_CO WITH mco
SELECT 0
USE rpt_rqst ALIAS rqst INDEX rpt_rqst
DO WHILE .T.
  GOTO TOP
  IF p_level!="a"
    rep_spcs=rpt_spcs   && replace memvar with memo field of parameters
                        && to pass to the report driver
    mdl=ALLTRIM(prg_mod)
                        && replace memvar with report driver
    mdata=data_path     &&   "       "     "   data path
    REPLACE p_level WITH "a"
                        && make record available to the request generator
    cx='EXECUTING '+mdl
    @ ROW()+2,39-LEN(cx)/2 SAY cx
    printed=.F.
    DO &mdl WITH &rep_spcs        && &mdl is a module in RPT_SERV.PRG
    IF printed                    && did we return successfully?
      SELECT rqst
    ELSE
      EXIT
    ENDIF
    CLEAR
    DO SHOW_CO WITH mco
  ENDIF
ENDDO
SET CURSOR ON

*************************************************************************
PROCEDURE SHOW_CO
PARAMETER company
CLEAR
cx=company + [ REPORT SERVER   --  PRESS "ESC" TO TERMINATE ]
SET INTENSITY ON
@  1 , 39 - LEN( cx ) / 2 GET cx
cx = " COPYRIGHT (c) 1994 COMPUTER PRODUCTS implementation "
@  3, 39 - LEN ( cx ) / 2 GET cx
CX = " ALL RIGHTS RESERVED WORLDWIDE "
@  5, 39 - LEN ( cx ) / 2 GET cx
SET INTENSITY OFF
CLEAR GETS
SET CURSOR OFF

*************************************************************************
PROCEDURE CST_LST && Print customer list.                               *
*************************************************************************
*  This is an example of a report driver.  There must be a custom coded *
*  report driver for each type of report the server will run.           *
*                                                                       *
*  This customer list can be sequenced by customer name or customer     *
*  number. It can be the entire list or for a chosen territory.         *
*                                                                       *
PARAMETER terr, seq     && terr = sales territory (0 for all)           *
*                       && seq is 1 for sequence by customer number     *
*                       && or 2 for  sequence by company name           *
*                                                                       *
SELECT 0
USE cstmers ALIAS cst
IF seq = 1
  SET ORDER TO cust_no
ELSE
  SET ORDER TO company
ENDIF
mord=IIF ( seq = 1 , 'CUST_NO' , 'COMPANY' )
IF terr > 0
  SELECT * FROM cst INTO CURSOR cust WHERE sales = terr ORDER BY &mord
ELSE
  USE cstmers ALIAS cust ORDER &mord && reopen with different alias
ENDIF
CLEAR
SET DEVICE TO PRINT
rdate = DTOC ( DATE ( ) )
rtime = TIME ( )
pg = 0        && define page and line numbers
ln = 66
DO WHILE NOT EOF ( )
  IF ln > 56
    IF pg > 0
      EJECT
    ENDIF
    pg = pg + 1
    @  0,5  SAY 'File cst_lst           CUSTOMER LIST' + SPACE ( 19 ) + rdate
    cx = LTRIM ( STR ( pg , 5 ) )
    @  1,5  SAY 'Page ' + cx + SPACE ( 5 - LEN ( cx ) ) + SPACE ( 8 ) + ;
      " RANKED BY " + IIF ( seq = 1 , "CUSTOMER NUMBER", "COMPANY NAME") + ;
      SPACE ( IIF ( seq = 1 , 11 , 14 ) ) + rtime
    @  3,5  SAY 'NUMBER    NAME   ADDRESS                 S TERR'
    ln = 5
  ENDIF
  @ ln , 5  SAY cust_no
  @ ln , 14 SAY company
  @ ln , 46 SAY sales
  IF addr1 != SPACE ( 25 )
    ln = ln + 1
    @ ln , 14 SAY addr1
  ENDIF
  IF addr2 != SPACE ( 25 )
    ln = ln + 1
    @ ln , 14 SAY addr2
  ENDIF
  ln = ln + 1
  @  ln , 14 SAY RTRIM ( city ) + ", " + state + " " + zip
  ln = ln + 2
  SKIP
ENDDO
EJECT           && needed on some printers to force last line to print.
USE             && close report file
IF terr > 0
  SELECT cst    && customer file
  USE           && close it: cannot CLOSE DATABASES because rpt_rqst.dbf
ENDIF           && must remain open!
**^^**  Never NEVER issue the command CLOSE DATABASES at the conclusion
**^^**  of a report driver, because rpt_rqst.dbf must remain open for the
**^^**  main program!
SET DEVICE TO SCREEN
printed = .T.   && let main program know that report was
                && successfully executed.
RETURN

*************************************************************************
PROCEDURE on_escp && for a graceful exit.                               *
*  revised       1/14/94. JFF.                                          *
*************************************************************************
SET CURSOR ON
devi = SET ( "DEVICE" )
SET DEVICE TO SCREEN
DEFINE WINDOW escp_key FROM 10,10 TO 16,55 TITLE ;
  ' "Esc" KEY HAS BEEN PRESSED ' SHADOW COLOR +W/G,N/W,N/R,W+*/N
ACTIVATE WINDOW escp_key
conf = SET("CONFIRM")
SET CONFIRM OFF
ch = " "
@  1,1  SAY [Press "S" to Suspend, "C" to Continue,]
@  3,1  SAY ["R" to return to FoxPro, or "Q" to quit ] ;
   GET ch PICTURE "!" VALID ch $ "CQRS"
READ
SET CONFIRM &conf
RELEASE WINDOW escp_key
IF ch = "S"
  SUSPEND
ELSE
  IF ch = "C"
    SET DEVICE TO &devi
    RETRY
  ELSE
    IF ch = "R"
      CLEAR WINDOWS
      CLOSE ALL
      CLEAR ALL
      SET STATUS ON
      SET TALK ON
      CANCEL
    ELSE
      QUIT
    ENDIF
  ENDIF
ENDIF

*****       eof: rpt_serv.prg                                       *****

